#!/bin/bash
# script d'archivage des logs de plus de X Mio, avec vidage des logs et rotation des archives  
# paramètres du script
rep_log="/var/log";
rep_arch="/media/KastorPollux/Software/Linux/log";
fichier_liste="/tmp/liste_temp_logrotate"; # gestion des fichiers avec espaces via mise en liste, sinon il faudrait 2 scripts, un de recherche et un de traitement en -exec du find 
taille_limite=5; # en Mioi
delai=15; # délai en jour avant effacement des anciennes archives
# fin des paramètres
# fonction pour les erreurs fatales
erreur(){
	echo "ERREUR : $1">&2;
	exit $2
}
if [ -d "$rep_log" -a -r "$rep_log" ] 
then
	# repertoire "$rep_log" existe en tant que repertoire, et on peut le lire
	if [ ! -d "$rep_arch" ] 
	then
		mkdir -p "$rep_arch"
		# si le mkdir ne signale pas d'erreur
		if [ ! $? -eq 0 ]
		then
			erreur "Le répertoire $rep_arch n'a pu être créé" 82
		fi 
	fi
	if [ ! -w "$rep_arch" ]
	then
		erreur "Le repertoire $rep_arch n'est pas modifiable" 81
	fi
	rep_liste=$(dirname "$fichier_liste");
	if [ ! -w "$rep_liste" ]
	then
		erreur "Le repertoire $rep_liste n'est pas modifiable, impossible de créer la liste" 83
	fi
	echo "Nettoyage des logs.";
	# comptage de la taille avant
	taille_avant=$(du -sh "$rep_log" | awk '{ print $1 }');
	# recherche des fichiers de + de X Mio dans $rep_log et archivage par tar / gz (extension tgz) dans $ep_arch
	find "$rep_log" -size +${taille_limite}M 2>/dev/null 1>"$fichier_liste"; # le fichier contient des lignes de type "$rep_log/nom fichier"
	while read ligne; 
	do 
		fichier=$(basename "$ligne"); # extraction du nom du fichier (sans arborescence) dans la ligne 
		repertoire=$(dirname "$ligne"); # extraction du repertoire parent
		if [ -f "$rep_arch/$fichier.tgz" ]
		then
			# si une archive existe déjà
			for ((suffixe=9;suffixe>=2;suffixe--)){
				# on enlève l'archive 9 si elle existe
				if [ -f "$rep_arch/$fichier.tgz.$suffixe" -a $suffixe -eq 9 ]
				then
					rm -f "$rep_arch/$fichier.tgz.$suffixe";
				fi
				# puis on augmente le numéro des anciennes archives (de 8 à 1) si elles existent
				precedent=$(expr $suffixe - 1);
				if [ -f "$rep_arch/$fichier.tgz.$precedent" ]
				then
					mv "$rep_arch/$fichier.tgz.$precedent" "$rep_arch/$fichier.tgz.$suffixe"
				fi
			}
			# puis on renomme l'ancienne archive en 1
			mv "$rep_arch/$fichier.tgz" "$rep_arch/$fichier.tgz.1"
		fi
		# une fois l'ancienne archive renommée
		tar -czf "$rep_arch/$fichier.tgz" "$ligne" 2>/dev/null;
		# si le tar ne signale pas d'erreur
		if [ $? -eq 0 ]
		then
			# si le fichier est w
			if [ -w "$fichier" ] 
			then
				# vidage du fichier log
				cat /dev/null > "$fichier"
			else
				if [ -w "$repertoire" ]
				then
					# suppr du fichier original
					rm -f "$ligne";
				else
					echo "le fichier $ligne n'a pu être vidé ni effacé" >&2; # affichage d'erreur non fatale
				fi
			fi
		else
			echo "le fichier $ligne n'a pu être archivé" >&2 # affichage d'erreur non fatale
		fi
	done<$fichier_liste
	# suppr du fichier temporaire de liste
	rm -f $fichier_liste
	taille_apres=$(du -sh "$rep_log" | awk '{ print $1 }');
	# nettoyage des anciennes archives
	find "$rep_arch" -mtime +$delai -type f -exec rm -f {} \;
	echo -e "Fin de nettoyage des logs\nAvant : $taille_avant, Apres : $taille_apres \n============================================";
else
	erreur "Le repertoire $rep_log n'existe pas ou n'est pas lisible" 80
fi
exit 0
